@@ -640,6 +640,16 @@ class ESP32FirmwareImage(BaseFirmwareImage):
                     flash_segments.insert(0, segment)
                     break
 
+            # For the bootloader image
+            # move ".dram0.bootdesc" segment to the top of the ram segment
+            # So bootdesc will be at the very top of the binary at 0x20 offset
+            # (in the first segment).
+            for segment in ram_segments:
+                if segment.name == ".dram0.bootdesc":
+                    ram_segments.remove(segment)
+                    ram_segments.insert(0, segment)
+                    break
+
             # check for multiple ELF sections that are mapped in the same
             # flash mapping region. This is usually a sign of a broken linker script,
             # but if you have a legitimate use case then let us know
@@ -803,13 +813,6 @@ class ESP32FirmwareImage(BaseFirmwareImage):
         self.min_rev_full = fields[6]
         self.max_rev_full = fields[7]
 
-        # reserved fields in the middle should all be zero
-        if any(f for f in fields[8:-1] if f != 0):
-            print(
-                "Warning: some reserved header fields have non-zero values. "
-                "This image may be from a newer esptool.py?"
-            )
-
         append_digest = fields[-1]  # last byte is append_digest
         if append_digest in [0, 1]:
             self.append_digest = append_digest == 1
