@@ -24,7 +24,7 @@ from .reset import (
     UnixTightReset,
 )
 from .util import FatalError, NotImplementedInROMError, UnsupportedCommandError
-from .util import byte, hexify, mask_to_shift, pad_to
+from .util import byte, hexify, mask_to_shift, pad_to, strip_chip_name
 
 try:
     import serial
@@ -97,13 +97,13 @@ DEFAULT_CONNECT_ATTEMPTS = cfg.getint("connect_attempts", 7)
 # Number of times to try writing a data block
 WRITE_BLOCK_ATTEMPTS = cfg.getint("write_block_attempts", 3)
 
-STUBS_DIR = os.path.join(os.path.dirname(__file__), "./targets/stub_flasher/")
+STUBS_DIR = os.path.join(os.path.dirname(__file__), "targets", "stub_flasher")
 
 
 def get_stub_json_path(chip_name):
-    chip_name = re.sub(r"[-()]", "", chip_name.lower())
+    chip_name = strip_chip_name(chip_name)
     chip_name = chip_name.replace("esp", "")
-    return STUBS_DIR + "stub_flasher_" + chip_name + ".json"
+    return os.path.join(STUBS_DIR, f"stub_flasher_{chip_name}.json")
 
 
 def timeout_per_mb(seconds_per_mb, size_bytes):
@@ -295,6 +295,7 @@ class ESPLoader(object):
             "flash_id": None,
             "chip_id": None,
             "uart_no": None,
+            "usb_pid": None,
         }
 
         if isinstance(port, str):
@@ -475,6 +476,9 @@ class ESPLoader(object):
             self.sync_stub_detected &= val == 0
 
     def _get_pid(self):
+        if self.cache["usb_pid"] is not None:
+            return self.cache["usb_pid"]
+
         if list_ports is None:
             print(
                 "\nListing all serial ports is currently not available. "
@@ -494,16 +498,19 @@ class ESPLoader(object):
         if active_port.startswith("/dev/") and os.path.islink(active_port):
             active_port = os.path.realpath(active_port)
 
+        active_ports = [active_port]
+
         # The "cu" (call-up) device has to be used for outgoing communication on MacOS
         if sys.platform == "darwin" and "tty" in active_port:
-            active_port = [active_port, active_port.replace("tty", "cu")]
+            active_ports.append(active_port.replace("tty", "cu"))
         ports = list_ports.comports()
         for p in ports:
-            if p.device in active_port:
+            if p.device in active_ports:
+                self.cache["usb_pid"] = p.pid
                 return p.pid
         print(
-            "\nFailed to get PID of a device on {}, "
-            "using standard reset sequence.".format(active_port)
+            f"\nFailed to get PID of a device on {active_port}, "
+            "using standard reset sequence."
         )
 
     def _connect_attempt(self, reset_strategy, mode="default_reset"):
