@@ -15,7 +15,9 @@ class ESP32S3BETA2ROM(ESP32S3ROM):
     EFUSE_BASE = 0x6001A000  # BLOCK0 read base address
 
     def get_chip_description(self):
-        return "ESP32-S3(beta2)"
+        major_rev = self.get_major_chip_version()
+        minor_rev = self.get_minor_chip_version()
+        return f"{self.CHIP_NAME} (revision v{major_rev}.{minor_rev})"
 
 
 class ESP32S3BETA2StubLoader(ESP32S3BETA2ROM):
@@ -33,6 +35,7 @@ class ESP32S3BETA2StubLoader(ESP32S3BETA2ROM):
         self.secure_download_mode = rom_loader.secure_download_mode
         self._port = rom_loader._port
         self._trace_enabled = rom_loader._trace_enabled
+        self.cache = rom_loader.cache
         self.flush_input()  # resets _slip_reader
 
 
